/** \file display.c
 * Functions for high level display.
 */

#include "display.h"
#include "lcd.h"
#include "board.h"
#include "string.h"
#include "ui.h"
#include "vs10xx.h"

xdata unsigned char displayTimeCount;
xdata struct displayItemStruct {
  char style;
  char title[12];
  char text[12];
} displayItem[DISPLAY_ITEMS];

void SetDisplayTitle(char n, char *s){
  char c; 
  for (c=0; c<12; c++) displayItem[n].title[c] = s[c];
}

void SetDisplaySignedValue(char n, int i){
  bit negative;
  char c;
  negative = 0;
  if (i<0){
    negative = 1;
    i = -i;
  }
  for (c=0; c<12; c++) displayItem[n].text[c] = ' ';
  c=2;
  if (i>99) c=3;
  if (i>999) c=6;
  do {
    displayItem[n].text[c--] = '0' + (i % 10);
    i /= 10; 
  }  while (i);
  if (negative){
    displayItem[n].text[c] = '-';
  } else {
    displayItem[n].text[c] = '+';
  }      
}

void SetDisplayText(char n, char *s){
  char c;
  for (c=0; c<12; c++) displayItem[n].text[c] = s[c];
}

void SetDisplayPercent (char n, char displayValue){
    unsigned char temp,majorp, minorp;
    if (displayValue>100){
      displayValue=100;
    }
    majorp = (displayValue-4) / 12;
    minorp = ((displayValue-4) % 12) >> 1;
    for (temp=0; temp<8; temp++){
      if (majorp>temp){
	displayItem[n].text[temp] = 4;
      }else{
	if (majorp<temp){
	  displayItem[n].text[temp] = ' ';
	}else{
	  switch(minorp){
	  case 0:
	    displayItem[n].text[temp] = ' ';
	    break;
	  case 1:
	    displayItem[n].text[temp] = 0;
	    break;
	  case 2:
	    displayItem[n].text[temp] = 1;
	    break;
	  case 3:
	    displayItem[n].text[temp] = 2;
	    break;
	  case 4:
	    displayItem[n].text[temp] = 3;
	    break;
	  case 5:
	    displayItem[n].text[temp] = 4;
	    break;
	  }//switch
	}
      }
    }       
    displayItem[n].text[8] = ' ';
    displayItem[n].text[9] = ' ';
    if (displayValue>99) displayItem[n].text[9]='1';
    displayItem[n].text[10] = ' ';
    if (displayValue>9) displayItem[n].text[10]='0' + ((displayValue / 10) % 10);
    displayItem[n].text[11]='0' + (displayValue % 10);
    
}



extern bit lyricsDisplay;
extern xdata unsigned char lyrics[40];
unsigned char PlayerTaskHandler();


/** Paint display */
void UpdateDisplay(unsigned char n){
  char i;
  static xdata char poly[16];
  
  PlayerTaskHandler();
  LcdLocateHome();
 
  if (n<1) {
#if 0
    LcdPutConstantString("  *VS1103*  ");
#else
    for (i=0; i<15; i++){
      unsigned char nn;
      temp.i = 1641 + i*19;
      Mp3WriteRegister(SCI_WRAMADDR, temp.b.b1, temp.b.b0);
      nn = Mp3ReadRegister(SCI_WRAM);
      if (nn){	
	if (nn>poly[i]){
	  LcdPutChar(0xff);
	} else {
	  LcdPutChar('0'+((i+1)%10));
	}      
      }else{
	LcdPutChar(' ');
      }
      poly[i]=nn;
    }   
    LcdPutConstantString("    ");
#endif    
  }else{
    for (i=0; i<12; i++){
      PlayerTaskHandler();
      LcdPutChar(displayItem[n-1].title[i]);
    }
  }

  if (displayTimeCount & 0x10){
    //LcdPutConstantString("\xa2\xff");
    LcdPutConstantString(" \xff");
  }else{
    //LcdPutConstantString("\xa2 ");
    LcdPutConstantString("  ");
  }

  for (i=0; i<12; i++){
    PlayerTaskHandler();
    LcdPutChar(displayItem[n].title[i]);
  }
  //LcdPutConstantString(" \x7c");
  LcdPutConstantString("  ");

  if (n==0) {
    LcdPutConstantString("     ");
  }else{
    if (n>UI_END_OF_MODES-2) {
      LcdPutConstantString("            ");
    }else{
      for (i=0; i<12; i++){
	LcdPutChar(displayItem[n+1].title[i]);
      }
    }
  }


  PlayerTaskHandler();
  LcdLocateLine2();
  PlayerTaskHandler();


  if (lyricsDisplay){
    for (i=0; i<40; i++){
      PlayerTaskHandler();
      LcdPutChar(lyrics[i]);
    }
  } else {
    
    if (n<1) {


#if 1
      LcdPutConstantString("^Channel Activity^ ");
#else
    for (i=8; i<16; i++){
      unsigned char n;
      temp.i = 1641 + i*19;
      Mp3WriteRegister(SCI_WRAMADDR, temp.b.b1, temp.b.b0);
      n = Mp3ReadRegister(SCI_WRAM);
      if (n>poly[i]){
	LcdPutChar('0'+((i+1)%10));
      }else{
	LcdPutChar(' ');
      }
      poly[i]=n;
    }   
    LcdPutConstantString("    ");
#endif    

    }else{
      for (i=0; i<12; i++){
	PlayerTaskHandler();
	LcdPutChar(displayItem[n-1].text[i]);
      }
    }
    if (displayTimeCount & 0x10){
      //    LcdPutConstantString("\x7c ");
      LcdPutConstantString("  ");
    }else{
      //    LcdPutConstantString("\x7c\xff");
      LcdPutConstantString(" \xff");
    }
    
    
    
    for (i=0; i<12; i++){
      PlayerTaskHandler();
      LcdPutChar(displayItem[n].text[i]);
    }
    //LcdPutConstantString(" \xa3");
    LcdPutConstantString("  ");
    if (n==0) {
      LcdPutConstantString("..-->");
    }else{
      if (n>UI_END_OF_MODES-2) {
	LcdPutConstantString("            ");
      }else{
	for (i=0; i<12; i++){
	  PlayerTaskHandler();
	  LcdPutChar(displayItem[n+1].text[i]);
	}
      }
    }
  }  

}
