/* VS1103 Karaoke Appliaction 26.9.2007 Panu-Kristian Poiksalo */
/* Copyright 2007 VLSI Solution Oy Tampere Finland */

/**\file player.c
 * VLSI Karaoke Demo
 */

/** \mainpage
    
This is VS1103B Karaoke Demo Source is based on VLSI Modular Player source.
For reference, see modularplayer source code at
http://www.vlsi.fi/player_vs1011_1002_1003/modularplayer/index.html

Some startup pointers:

- player.c is the main file. 
- The data transfer loops are done mainly in assembly to get them
  quick. C versions coexist with the asm versions.
- AvailableProcessorTime() does the user interface
- MMC/SD cars are supported. SDHC would need a small change in addressing.
- Recording is done only to first R*.WAV file on either disk. Only the first fragment is recorded.



Contact us at mp3@vlsi.fi

Good Luck!
 
*/


#include "board.h"
#include "lcd.h"
#include "filesys.h"
#include "vs10xx.h"
#include "storage.h"
#include "display.h"
#include "string.h"
#include "ui.h"
#include "mmc.h"
#include "record.h"

/**  typedef for loading patch code */
#define u_int8 code unsigned char 
/**  typedef for loading patch code */
#define u_int16 code unsigned int 
#include "miditext1103.ctab-installed"
#define CODELEN CODE_SIZE

/** How much to wait for button pressed before returning to title */
#define UI_RETURN_DELAY 150
/** Playing State Global */
xdata playingstatetype playingState = PS_NORMAL;

/** Interrupt-driven flag to reduce unnecessary UI updates.
 * This is set to 1 by timer interrupt every n milliseconds. */
bit userInterfaceDelayElapsed = 0;
bit lyricsDisplay;


/** Current User Interface mode (Title, Volume, ...) */
xdata uimodetype uiMode = 0;
xdata unsigned char SPMax=0;

xdata int currentWaveFile;
xdata int currentMidiFile;
xdata int currentRecordFile;
xdata char lyrics[40];

/** - <b>volume</b>: Current volume setting */
xdata unsigned char volume ;
/** - <b>bass</b>: Current bass enhancer setting */
xdata unsigned char bass;
/** - <b>treble</b>: Current treble enhancer setting */
xdata unsigned char treble = 64; //middle setting = 0
/** - <b>reclevel</b>: Current recording level setting */
xdata unsigned char reclevel = 50; 
xdata signed char transpose = 0;
xdata unsigned int midiChannelMask;

xdata unsigned char currentWaveVolume;
xdata unsigned char currentMidiVolume;
xdata unsigned char currentMicVolume;
xdata unsigned char currentMicGainSetting;

bit midiSciActive;
bit midiDisk;
xdata unsigned char midiCurrentFragment;
xdata unsigned long midiCurrentSector;
xdata unsigned long midiSectorsLeftInFragment;

bit waveSdiActive;
bit waveDisk;
xdata unsigned char waveCurrentFragment;
xdata unsigned long waveCurrentSector;
xdata unsigned long waveSectorsLeftInFragment;

bit recordActive;
bit recordDisk;
//xdata unsigned char recordCurrentFragment; //can be only 1 fragment
xdata unsigned long recordCurrentSector;
xdata unsigned long recordSectorsLeftInFragment;

bit echoActive;





/** User Interface Timer interrupt.
 * Does NOT do any screen/button handling, only updates
 * screen scroll variable and the 
 * userInterfaceDelayElapsed flag. */
void timer0_interrupt(void) interrupt 1{
  //LED4 = LED_ON;
  userInterfaceDelayElapsed = 1;
  displayTimeCount++;
  if (SP>SPMax) SPMax = SP;
  //LED4 = LED_OFF;
}

data signed char rotaryValue;
// Rotary Encoder Interrupt
void timer1_interrupt(void) interrupt 3{
  static bit ledi;  
  P3_2 = 1;
  P3_4 = 1;
  P3_5 = 1;

  if (P3_4) {
    rotaryValue++;
  }else{
    rotaryValue--;
  }
  ledi = !ledi;
  //LED4 = ledi;
  TF1 = 0;
}


/// Display a message box to user and prompt for key press
void MessageBox(char *Msg){
  LcdReset();
  LcdPutString(Msg);
  LcdLocateLine2();
  LcdPutConstantString("                                  [~ OK]");
  while (KEY_RIGHT)
    ;
  while (!KEY_RIGHT)
    ;
  LcdReset();
}



/** Load a patch code to VS10xx */
void LoadPatch(){
#if 1
  ConsoleWrite ("Loading patch.\r");
  for (temp.i=0; temp.i < CODELEN; temp.i++) {    
    while (!MP3_DREQ)
      ;
    Mp3WriteRegister(atab[temp.i], dtab[temp.i]>>8, dtab[temp.i]&0xff);
    while (!MP3_DREQ)
      ;
   }
  Delay(10);
#endif
}

/** Start patch code */
void StartPatch()
{
  ConsoleWrite("Start Patch.\r");
    while (!MP3_DREQ)
      ;
#if 1
  Mp3WriteRegister(SCI_AIADDR, 0x00, 0x30);
    while (!MP3_DREQ)
      ;
#endif
}





/// Check status of buffers and do one disk block transfer
unsigned char PlayerTaskHandler() {  
  //_____________________________________________________
  // WAV playing ----------------------------------------
  //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


  // Highest priority: WAVE IN stream is SDI stream in
  // Check whether SDI in stream has space for 1 disk sector
  if ((waveSdiActive) && (MP3_DREQ)){    
    GREEN_LED=LED_ON;

    SelectDisk(waveDisk);
    TransferSectorSDI(waveCurrentSector); //Send WAV to VS1103

    // Find next sector or end of file
    waveCurrentSector++;
    if (waveSectorsLeftInFragment-- == 0){
      waveCurrentFragment++;
      waveCurrentSector = waveFragment[waveCurrentFragment].start;
      waveSectorsLeftInFragment = waveFragment[waveCurrentFragment].length;
      if (waveSectorsLeftInFragment == 0) {
	waveSdiActive = 0;
      }      
    }
    GREEN_LED=LED_OFF;
    return 1;
  }



  //_____________________________________________________
  //Recording -------------------------------------------
  //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


  if ((recordActive) && ((Mp3ReadRegister(SCI_IN1) >> 8) >= 0x20)){
    //Sector is available for storing
    
    // EA=0;
    // BlockReceive in C
    dataBufPtr = diskSect.raw.buf;
    // Use global pointer and unrolled loop for speed
    while (dataBufPtr < diskSect.raw.buf+512){


      // Below is the spi data transfer code in ASSEMBLER 
      // (for super fast data read from VS1103) and also in C.

      // READ ONE SECTOR (512 bytes) FROM VS1103 REC BUFFER
      // TO DISKSECT.RAW.BUF[512]

#if 1
_asm

  push dpl
  push dph
  push psw
  push acc
  mov	dpl,_dataBufPtr
  mov	dph,(_dataBufPtr + 1)


  // CHIP SELECT
  clr	_P2_3
  mov	_SPDAT,#0x03
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	_SPDAT,#0x08
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	_SPDAT,#0xFF
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  mov	a,_SPSTA
  mov	a,_SPDAT
  movx	@dptr,a
  inc dptr

  // CHIP DESELECT
  setb	_P2_3
  
  mov	_dataBufPtr, dpl
  mov	(_dataBufPtr + 1), dph
  pop acc
  pop psw
  pop dph
  pop dpl


_endasm;

#else
      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


      Mp3SelectControl();
      SPIPutCharWithoutWaiting(VS_READ_COMMAND);SPIWait();
      SPIPutCharWithoutWaiting(SCI_IN0);SPIWait();
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      SPIPutCharWithoutWaiting(0xff); SPIWait(); *dataBufPtr++ = SPI_RESULT_BYTE;
      Mp3DeselectControl();


#endif

    }
    //ConsolePutChar('w');

    EA=1;

    
    RED_LED = LED_ON;
    sectorAddress.l = recordCurrentSector;
    recordCurrentSector++;

    SelectDisk(recordDisk);
    WriteDiskSector(sectorAddress.l);
    RED_LED = LED_OFF;
    
    recordSectorsLeftInFragment--;
    if (recordSectorsLeftInFragment<1) recordActive = 0;
    
    return 2;

  }

  //_____________________________________________________
  //   ECHO   -------------------------------------------
  //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


  if ((echoActive  ) && ((Mp3ReadRegister(SCI_IN1) >> 8) >= 0x20)){
    
    // EA=0;
    // BlockReceive in C
    dataBufPtr = diskSect.raw.buf;
    // Use global pointer and unrolled loop for speed
    while (dataBufPtr < diskSect.raw.buf+256){
   
      //Similar fast transfer than in the recording mode 
      // -> record and send back to VS1103 to make echo from the microphone
   
_asm

  push dpl
  push dph
  push psw
  push acc
  mov	dpl,_dataBufPtr
  mov	dph,(_dataBufPtr + 1)


  // CHIP SELECT
  clr	_P2_3
  mov	_SPDAT,#0x03
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	_SPDAT,#0x08
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	_SPDAT,#0xFF
  mov	a,_SPSTA
  jnb     acc.7,.-4
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  // CHIP DESELECT
  setb	_P2_3
  // CHIP SELECT
  clr	_P2_3
  mov	a,_SPDAT
  mov	_SPDAT,#0x03
  movx	@dptr,a
  mov	_SPDAT,#0x08
  inc dptr  
  mov	a,_SPSTA
  mov	_SPDAT,#0xFF
  nop
  nop
  nop
  mov	a,_SPSTA
  mov	a,_SPDAT


  mov	_SPDAT,#0xFF
  movx @dptr,a
  inc dptr
  mov	a,_SPSTA
  mov	a,_SPDAT
  movx	@dptr,a
  inc dptr

  // CHIP DESELECT
  setb	_P2_3
  
  mov	_dataBufPtr, dpl
  mov	(_dataBufPtr + 1), dph
  pop acc
  pop psw
  pop dph
  pop dpl


_endasm;

 
    }
    //ConsolePutChar('w');
    
    EA=1;
    

    //while (MP3_DREQ){
    
      Mp3SelectData();
      // BlockTransmit in C
      dataBufPtr = diskSect.raw.buf;
      // Use global pointer and unrolled loop for speed
      while (dataBufPtr < diskSect.raw.buf+256){
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
	SPIPutCharWithoutWaiting(*dataBufPtr++); 
      }
      Mp3DeselectData();
      //}
    return 2;

  }


  //_____________________________________________________
  // MIDI playing ---------------------------------------
  //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

  
  // MIDI IN stream is SCI stream in
  // Check whether SCI in stream has space for 1 disk sector
  if (midiSciActive && (Mp3ReadRegister(SCI_IN1) & 0xff) >= 0x20){
    LED1 = LED_ON;

    SelectDisk(midiDisk);
    TransferSectorSCI(midiCurrentSector); //Send MID to VS1103

    // Find next sector or end of file
    midiCurrentSector++;
    if (midiSectorsLeftInFragment-- == 0){
      midiCurrentFragment++;
      midiCurrentSector = midiFragment[midiCurrentFragment].start;
      midiSectorsLeftInFragment = midiFragment[midiCurrentFragment].length;
      if (midiSectorsLeftInFragment == 0) {
	midiSciActive = 0;
      }      
    }

    LED1=LED_OFF;
    return 3;
  }
    
  return 0; //No transfer was pending.
}


/// Stop recording
void CancelRecord(){
  if (recordActive || vs1103_recording_active){
    
    echoActive = 0;
    recordActive = 0;
    vs1103_recording_active = 0;
    Vs1103SetMode();
    SetDisplayText(UI_ECHOMODE, "Play/Record ");

    currentRecordFile = 0;
    SetDisplayText(UI_RECORDFILE,"----        ");
    

  }
}


/// Stop wave file playing
void CancelWave(){
  if (waveSdiActive){
    Vs1103CancelWave();
  }
  waveSdiActive = 0;
  SetDisplayText(UI_WAVEFILE,"----        ");
}

/// Stop MIDI file playing
void CancelMidi(){
  if (midiSciActive){
    Vs1103CancelMidi();
  }
  midiSciActive = 0;
  SetDisplayText(UI_MIDIFILE,"  ----      ");
}


/// Route MIDI from UART in realtime mode
void SetMidiFromUART(){   
  if (!vs1103_real_time_midi){
    if (recordActive){
      CancelRecord();
      MessageBox("RTMIDI: Recording Cancelled (N1).");
    }
    if (waveSdiActive){
      MessageBox("RTMIDI: Wave Playing Cancelled (N1).");
      CancelWave(); //UART/SCI transition forces reset      
    }
    CancelMidi();
    vs1103_real_time_midi = 1;
    LoadPatch();
    Vs1103SetMode();
  }
  SetDisplayText(UI_MIDISOURCE,"MIDI Input  ");

}

/// Route MIDI from SCI in file play mode
void SetMidiFromFile(){
  if (vs1103_real_time_midi){
    if (recordActive){
      CancelRecord();
      MessageBox("RTMIDI: Recording Cancelled (N1).");
    }
    if (waveSdiActive){
      MessageBox("RTMIDI: Wave Playing Cancelled (N1).");
      CancelWave(); //UART/SCI transition forces reset      
    }
    vs1103_real_time_midi = 0;
    Vs1103SetMode();
    SetDisplayText(UI_MIDISOURCE,"File (Fmt 0)");
  }
}

/// Enable delay echo. It's done by sending the mic recording
/// result ADPCM stream back to the WAVE IN port of VS1103
void EchoEnable(){
  CancelWave();
  CancelRecord();
  echoActive = 1;
  vs1103_recording_active = 1;
  vs1103_record_mixer = 1; //1 = reverb echo; 0 = delay echo
  Vs1103SetMode();  
  
  currentWaveVolume = 11;
  Mp3SetMixerVolume(currentMidiVolume, currentWaveVolume,
		    currentMicVolume);      
  SetDisplayPercent(UI_WAVEVOLUME, currentWaveVolume);
  SetDisplayText(UI_ECHOMODE, "Delay Echo  ");


  // MAKE A WAV HEADER AND TRANSMIT IT TO VS1103
  for (temp.c=0; temp.c<56; temp.c++){ //temp is unsafe global temp variable.
    //    diskSect.raw.buf[temp.c] = RIFFHeader9k6Hz[temp.c];
    diskSect.raw.buf[temp.c] = RIFFHeader0[temp.c];
  }
  for (temp.i=52; temp.i<504; temp.i++){
    diskSect.raw.buf[temp.i] = 0;
  }
  for (temp.i=504; temp.i<512; temp.i++){
    diskSect.raw.buf[temp.i] = RIFFHeader504[temp.i-504];
  }
  
  Mp3SelectData();
  // BlockTransmit in C
  dataBufPtr = diskSect.raw.buf;
  // Use global pointer and unrolled loop for speed
  while (dataBufPtr < diskSect.raw.buf+512){
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
    SPIPutCharWithoutWaiting(*dataBufPtr++); 
  }
  {
    int i;
    for (i=0; i<512; i++){
      SPIPutCharWithoutWaiting(0);
      SPIWait();
    }
  }

  Mp3DeselectData();
}

void EchoDisable(){
  echoActive = 0;
  vs1103_record_mixer = 1;
  CancelWave();
  CancelRecord();
}



unsigned char SelectMidiFile(int fileNumber){

  CancelMidi();
  SetMidiFromFile();
  if (fileNumber == 0) return 0;

  //ConsoleWrite("\rLooking for MIDI file\r");
  midiDisk = 0;
  if (fileNumber < 0) {
    midiDisk = 1;
    fileNumber = -fileNumber;
  }
  SelectDisk(midiDisk);
 
  if (!OpenFile(fileNumber,'M','I','D')){
    unsigned char i;
    SetDisplayText(UI_MIDIFILE,currentFileName);
    BuildFragmentTable();
    for (i=0; i<MAX_NUMBER_FRAGMENTS; i++){
      midiFragment[i].start = fragment[i].start;
      midiFragment[i].length = fragment[i].length;
    }
    midiCurrentSector = midiFragment[0].start;
    midiSectorsLeftInFragment = midiFragment[0].length;
    Vs1103B_SetChannelMask(midiChannelMask);
    midiSciActive = 1;
    if (midiSectorsLeftInFragment == 0) {
      midiSciActive = 0;
    }
    for (i=0; i<40; i++){
      lyrics[i] = ' ';
    }
    lyrics[34] = 'T';
    lyrics[35] = 'e';
    lyrics[36] = 'x';
    lyrics[37] = 't';
    lyrics[38] = ':';

    sectorAddress.l = midiCurrentSector;
    ReadDiskSector(sectorAddress.l);
    if (diskSect.raw.buf[9] != 0){
      MessageBox("Please convert file to MIDI format 0.");
    }

    #if 0
    for (i=0; i<16; i++){
      ConsolePutHex8(diskSect.raw.buf[i]);
      ConsolePutChar(' ');
      ConsolePutChar(diskSect.raw.buf[i]);
      ConsolePutChar(13);
    }
    #endif


    return 0;
  } else {
    SetDisplayText(UI_MIDIFILE,"  No file   ");
  }
  return 1;
}


unsigned char SelectRecordFile(int fileNumber){
  
  recordDisk = 0;
  if (fileNumber < 0) {
    recordDisk = 1;
    fileNumber = -fileNumber;
  }
  SelectDisk(recordDisk);

  if (fileNumber == 0){
    
    recordActive=0;    
    vs1103_recording_active = 0;
    Vs1103SetMode();
    SetDisplayText(UI_RECORDFILE,"----        ");
    return 0;
  }

  //ConsoleWrite("\rLooking for Wave file for recording\r");
  while (!OpenFile(fileNumber++,'W','A','V')){
    if (currentFileName[0]=='R'){
      SetDisplayText(UI_RECORDFILE,currentFileName);

      BuildFragmentTable();
      recordCurrentSector = fragment[0].start;
      recordSectorsLeftInFragment = fragment[0].length-1;      
      
      for (temp.c=0; temp.c<56; temp.c++){ //temp is unsafe global temp variable.
	diskSect.raw.buf[temp.c] = RIFFHeader0[temp.c];
      }
      for (temp.i=52; temp.i<504; temp.i++){
	diskSect.raw.buf[temp.i] = 0;
      }
      for (temp.i=504; temp.i<512; temp.i++){
	diskSect.raw.buf[temp.i] = RIFFHeader504[temp.i-504];
      }
      WriteDiskSector(recordCurrentSector++);
      echoActive = 0;
      vs1103_record_mixer = 1;
      recordActive = 1;
      vs1103_recording_active = 1;
      Vs1103SetMode();
      return 0;
    }
  }
  return 1;
}


unsigned char SelectWaveFile(int fileNumber){ 
  unsigned char i;
  
  waveDisk = 0;
  if (fileNumber < 0) {
    waveDisk = 1;
    fileNumber = -fileNumber;
  }
  SelectDisk(waveDisk);

  CancelWave();
  if (fileNumber == 0) return 0;

  if (!OpenFile(fileNumber,'W','A','V')){
    SetDisplayText(UI_WAVEFILE,currentFileName);
    if (waveSdiActive){ //End playback of previous file
      Vs1103CancelWave();
    }	       

    if (!recordActive && !midiSciActive){
      UpdateDisplay(uiMode);
    }

    BuildFragmentTable();
    for (i=0; i<MAX_NUMBER_FRAGMENTS; i++){
      waveFragment[i].start = fragment[i].start;
      waveFragment[i].length = fragment[i].length;
    }
    waveCurrentSector = fragment[0].start;
    waveSectorsLeftInFragment = fragment[0].length;
    waveSdiActive = 1;
    if (waveSectorsLeftInFragment == 0) {
      waveSdiActive = 0;
    }
    return 0; //OK
  }
  return 1; //ERROR
}




//  U S E R   I N T E R F A C E   C O D E
// ---------------------------------------

/** User interface.

 This function is called when the player is playing a song
 and there is free processor time. The basic task of this
 function is to implement the player user interface. */
void AvailableProcessorTime(){
  unsigned int i; //temp variable
  /// Bass/Treble settings have been changed
  static bit bassUpdateNeeded;
  /// Key has been pressed but not released
  static bit keyInTransit;
  

  if (!userInterfaceDelayElapsed){
    return; /* Not yet time to do user interface stuff */
  }
  userInterfaceDelayElapsed = 0;
  
  if (keyInTransit){ //Keypress already registered
    if (!KEY_UP && !KEY_DOWN && !KEY_LEFT && !KEY_RIGHT){
      keyInTransit = 0; //Wait until key not pressed
    }
  }else{ //Is there a new keypress?
    if (KEY_UP){
      rotaryValue = NEXT_THRESHOLD;
      keyInTransit = 1;
    }
    if (KEY_DOWN){
      rotaryValue = PREV_THRESHOLD;
      keyInTransit = 1;
    }

    if (KEY_LEFT){
      if (uiMode) {
	uiMode--;
	keyInTransit = 1;
	rotaryValue = 0;
      }
    }
    if (KEY_RIGHT){
      uiMode++;
      keyInTransit = 1;
      if (uiMode == UI_END_OF_MODES) {
	uiMode--;
      }
      rotaryValue = 0;
    }
    if (keyInTransit){
      //Always do this once when a key is pressed:
      LcdReset();
      lyricsDisplay = 0;      
    }
  }




  //  K E Y B O A R D   E V E N T S
  //  -----------------------------


  //   N  E  X  T
  //   N  E  X  T


  //Is there a large PLUS(UP) rotation? This can mean
  //a "NEXT" or "SWITCH ON" request from the user
  if (rotaryValue >= NEXT_THRESHOLD) {
    signed char storedRotaryValue = rotaryValue;


    // Do all midi ch on/off settings in one place
    if ((uiMode >= UI_MIDI_CH1) && (uiMode <= UI_MIDI_CH16)){
      midiChannelMask &= ~(1 << (uiMode - UI_MIDI_CH1));
      Vs1103B_SetChannelMask(midiChannelMask);
      SetDisplayText(uiMode,"On          ");
      rotaryValue = 0;

    }else{   
      rotaryValue = 0;
      
      //What parameters can be affected?
      switch(uiMode){
      case UI_MIDITRANSPOSE:
	transpose++;
	SetDisplaySignedValue(UI_MIDITRANSPOSE,transpose);
	Vs1103B_SetMidiTranspose(transpose);
	break;

	
      case UI_RECORDFILE:		
	if (currentRecordFile){
	  currentRecordFile=0;
	  CancelRecord();
	}else{
	  currentRecordFile = 0;
	  if (!SelectRecordFile(1)){
	    currentRecordFile = 1;
	  }
	}
	break;
	  
      case UI_WAVEFILE:	
	if (!SelectWaveFile(currentWaveFile+1)){
	  currentWaveFile++;
	}
	break;
	
      case UI_MIDIFILE:
	if (!SelectMidiFile(currentMidiFile+1)){
	  currentMidiFile++;
	}
	break;
	
      case UI_MIDISOURCE:
	SetMidiFromUART();
	break;

	
      case UI_ECHOMODE:
	EchoEnable();
	break;
	
      
      default:
	//The request does not apply to current uiMode
	rotaryValue = storedRotaryValue; //restore value
	break;
      }
    }
  }

  


  //   P  R  E  V  I  O  U  S
  //   P  R  E  V  I  O  U  S


  //Is there a large MINUS rotation? This can mean
  //a "PREVIOUS" or "SWITCH OFF" request from the user
  if (rotaryValue <= PREV_THRESHOLD) {
    signed char storedRotaryValue = rotaryValue;

    // Do all midi ch on/off settings in one place
    if ((uiMode >= UI_MIDI_CH1) && (uiMode <= UI_MIDI_CH16)){
      midiChannelMask |= (1 << (uiMode - UI_MIDI_CH1));
      Vs1103B_SetChannelMask(midiChannelMask);
      SetDisplayText(uiMode,"Off         ");
      rotaryValue = 0;

    }else{   
      rotaryValue = 0; //Store and clear
      //What parameters can be affected?


      switch(uiMode){


      case UI_WAVEFILE:	
	if (!SelectWaveFile(currentWaveFile-1)){
	  currentWaveFile--;
	}
	break;
	
      case UI_MIDIFILE:
	if (!SelectMidiFile(currentMidiFile-1)){
	  currentMidiFile--;
	}
	break;


      case UI_RECORDFILE:
	if (currentRecordFile){
	  currentRecordFile=0;
	  CancelRecord();
	}else{
	  currentRecordFile = 0;
	  if (!SelectRecordFile(-1)){
	    currentRecordFile = -1;
	  }
	}
	break;
	
      case UI_MIDITRANSPOSE:
	transpose--;
	SetDisplaySignedValue(UI_MIDITRANSPOSE,transpose);
	Vs1103B_SetMidiTranspose(transpose);
	break;

      case UI_MIDISOURCE:
	SetMidiFromFile();
	break;	
	
      case UI_ECHOMODE:
	EchoDisable();
	break;
	
      default:
	//The request does not apply to current uiMode
	rotaryValue = storedRotaryValue; //restore value
	break;
      }
    }
  }






  //  A  D  J  U  S  T 
  //  A  D  J  U  S  T 

  //Is there a plus/minus value from the rotary button?
  if (rotaryValue){
    lyricsDisplay = 0;

    //What parameter is affected by the rotary value?
    switch(uiMode){

    case UI_MIDIVOLUME:
      if (rotaryValue>0){
	if (currentMidiVolume<30){
	  currentMidiVolume++;
	}
      } else {
	if (currentMidiVolume){
	  currentMidiVolume--;
	}
      }
      Mp3SetMixerVolume(currentMidiVolume, currentWaveVolume,
			currentMicVolume);      
      SetDisplayPercent(UI_MIDIVOLUME, currentMidiVolume);
      rotaryValue = 0;
      break;
      
    case UI_WAVEVOLUME:
      if (rotaryValue>0){
	if (currentWaveVolume<30){
	  currentWaveVolume++;
	}
      } else {
	if (currentWaveVolume){
	  currentWaveVolume--;
	}
      }
      Mp3SetMixerVolume(currentMidiVolume, currentWaveVolume,
			currentMicVolume);      
      SetDisplayPercent(UI_WAVEVOLUME, currentWaveVolume);
      rotaryValue = 0;
      break;
      
    case UI_MIC_VOLUME:
      if (rotaryValue>0){
	if (currentMicVolume<30){
	  currentMicVolume++;
	}
      } else {
	if (currentMicVolume){
	  currentMicVolume--;
	}
      }
      Mp3SetMixerVolume(currentMidiVolume, currentWaveVolume,
			currentMicVolume);      
      SetDisplayPercent(UI_MIC_VOLUME, currentMicVolume);
      rotaryValue = 0;
      break;
      
    case UI_MIC_GAIN:
      if (rotaryValue>0){
	if (currentMicGainSetting<62){
	  currentMicGainSetting++;
	}
      } else {
	if (currentMicGainSetting){
	  currentMicGainSetting--;
	}
      }
      Vs1103SetMicGain(currentMicGainSetting);
      SetDisplayPercent(UI_MIC_GAIN, currentMicGainSetting);
      rotaryValue = 0;
      break;
      
    case UI_VOLUME:
      if (volume>=rotaryValue){
	volume-=rotaryValue;
      }
      rotaryValue = 0;
      Mp3SetVolume(volume,volume);
      if (volume<100){
	SetDisplayPercent(UI_VOLUME,(100-volume));
      }else{
	SetDisplayPercent(UI_VOLUME,0);
      }
      rotaryValue = 0;
      break;
      
      
    default:
      break;
    }    
  }//Switch

  PlayerTaskHandler();
  UpdateDisplay(uiMode);
  PlayerTaskHandler();

  
  if (bassUpdateNeeded){
    unsigned int newBassRegister;
    
    //User has pushed button to alter bass/treble register
    //calculate new value
    
    /** Bass/Treble setting calculation 
     *
     * There are 2 or 4 adjustments possible:
     * - Bass Frequency (1011B/E, 1002, 10x3) : 2..15 -> 20..1500 Hz
     * - Bass boost (1011B/E, 1002, 10x3) : 0..15 -> 0ff..+15dB)
     * - Treble Frequency (1011E, 10x3) : 0..15 -> 0..15kHz
     * - Treble level (1011E, 10x3) : -8..+7 -> -12..+10.5dB
     * 
     * In this software, all are adjusted based from a single setting
     * (0..128) for bass and another for treble. You may wish to allow
     * user to control frequency and level separately or set fixed
     * frequencies for a known speaker type. This algorithm increases
     * the frequency range of adjustment with the increasing level.
     */
    
    //Let's start from bass frequency setting.
    //min(0) should give 2 (20Hz), max(127) should give 15 (150Hz)
    newBassRegister = (bass + 23) / 10; //into bits 3..0, clear hibits
    
    //Bass boost level.
    //min(0) should give 0, max(127) should give 15
    newBassRegister |= (bass>>3)<<4; //insert to bits 7..4
    
    //Then the treble frequency
    //min(0) should give 15(15kHx), max(127) should give 2(2kHz)
    newBassRegister |= (((148-treble)>>3)+2)<<8; //insert into bits 11..8
    
    //Finally the treble value (-8..7)
    //min(0) should give -8, max(127) should give 7;
    newBassRegister |= ((treble>>3)-8)<<12; //insert into bits 15..12
    
    if (Mp3ReadRegister(SCI_BASS)!=newBassRegister){
      Mp3WriteRegister(SCI_BASS,newBassRegister>>8,newBassRegister&0xff);
    }
  }

}  

/** Program Entry Point.
 * In the final program, main() should be very small.
 * Currently it's the playground for developing the player
 * functionality. Most of it will be cleared away to new
 * functions soon. */
void main(){
  unsigned char i;
  P2_1 = 0;


  waveSdiActive = 0;
  midiSciActive = 0;
  recordActive = 0;
  echoActive = 0;
  InitBoard();

  ConsoleWrite("\rApplication Start.\r");
  P2_1 = 1;
  LcdReset();
  LcdReset();
  LcdLocateHome();
  LcdPutConstantString ("  VS1103B MIDI/ADPCM/Karaoke Chip Demo");
  LcdLocateLine2();
  LcdPutConstantString ("   2006 VLSI Solution Oy  www.vlsi.fi ");
  ConsoleWrite ("\rVLSI Karaoke Demo\rStarting up.\r\r");
  RED_LED = LED_ON;


  Mp3Reset();

 
  InitFileSystem();
 
  // INITIAL MODE AND USER INTERFACE TEXTS
  
  for (i=0; i<UI_END_OF_MODES; i++){
    SetDisplayTitle(i,"            ");
    SetDisplayText(i,"------      ");
  }

  //SetDisplayTitle(UI_MODE, "Current Mode");
  //SetDisplayText(UI_MODE,"Auto Play   ");

  SetDisplayTitle(UI_MIDISOURCE, "MIDI Source ");
  SetDisplayText(UI_MIDISOURCE,  "MIDI Input  ");

  SetDisplayTitle(UI_MIDIFILE, "MIDIf0 File ");
  SetDisplayText(UI_MIDIFILE," ----       ");


  SetDisplayTitle(UI_WAVEFILE, "Wave File   ");
  SetDisplayTitle(UI_RECORDFILE, "Record File ");

  SetDisplayTitle(UI_MIDITRANSPOSE, "TransposeMID");

  SetDisplayTitle(UI_MIDIVOLUME, "MIDI Volume ");
  SetDisplayTitle(UI_WAVEVOLUME, "Wave Volume ");
  SetDisplayTitle(UI_MIC_VOLUME, "Mic Volume  ");
  SetDisplayTitle(UI_MIC_GAIN, "Mic Gain    ");

  SetDisplayTitle(UI_VOLUME, "MasterVolume");

  SetDisplayTitle(UI_ECHOMODE, "Wave usage  ");
  SetDisplayText(UI_ECHOMODE,  "Play/Record ");

  SetDisplayTitle(UI_BASS, "Bass        ");
  SetDisplayTitle(UI_TREBLE, "Treble      ");

  for (i=0; i<40; i++){
    lyrics[i] = ' ';
  }
  lyrics[0] = 'M';
  lyrics[1] = 'i';
  lyrics[2] = 'd';
  lyrics[3] = 'i';

  lyrics[5] = 'C';
  lyrics[6] = 'h';
  lyrics[7] = '.';



  uiMode = 0;
  for (i=1; i<17; i++){
    lyrics[9] = '0' + (i / 10);
    lyrics[10] = '0' + (i % 10);
    SetDisplayTitle((UI_MIDI_CH1 - 1)+i, lyrics);
    SetDisplayText((UI_MIDI_CH1 - 1)+i, "On          ");

  }


  for (i=0; i<40; i++){
    lyrics[i] = ' ';
  }
  lyricsDisplay = 0; //Not now in lyrics mode

  transpose = 0;
  SetDisplaySignedValue(UI_MIDITRANSPOSE,transpose);
  volume = 8;
  SetDisplayPercent(UI_VOLUME,(100-volume));
  bass = 0;
  treble = 64;
  midiChannelMask = 0;

  currentMicGainSetting  = 25;
  vs1103_record_mixer = 1;
  vs1103_recording_active = 0; 
  vs1103_real_time_midi = 1;
  currentMidiFile = 0;
  currentWaveFile = 0;
  recordActive = 0;  
  Vs1103SetMicGain(25);
  Vs1103SetMode(); // Set Initial Mode
  currentWaveVolume = 23;
  currentMidiVolume = 23;
  currentMicVolume = 23;
  Mp3SetMixerVolume(currentMidiVolume, currentWaveVolume, currentMicVolume);
  SetDisplayPercent(UI_MIDIVOLUME, currentMidiVolume);
  SetDisplayPercent(UI_WAVEVOLUME, currentWaveVolume);
  SetDisplayPercent(UI_MIC_VOLUME, currentMicVolume);
  SetDisplayPercent(UI_MIC_GAIN, currentMicGainSetting);

  RED_LED = LED_OFF;

  // Start "User Interface" timer
  ET0 = 1;
  EA = 1; 
  TR0 = 1;


  // SYSTEM MAIN LOOP
  while(1){
    data unsigned int ilyric;

    while(PlayerTaskHandler())
      ; //Do all pending data transfers

    AvailableProcessorTime(); //User Interface Handler

    //Read Back Lyrics from the VS1103
    ilyric = Mp3ReadRegister(SCI_AICTRL3);
    if (ilyric){
      if (ilyric & 0xFF00U){
	//ConsolePutHex16(i);//Debug info
      }else{
	//Shift the lyrics display buffer 1 char to the left
	for (i=0; i<39; i++){
	  lyrics[i] = lyrics[i+1];
	}
	//Add a new lyrics character to the lyrics display buffer
	lyrics[39] = ilyric;
	lyricsDisplay = 1;
      }
    }
  }   
}


