/** \file vs10xx.h
 * Headers for interfacing with the mp3 player chip.
 * Interfacing the New Way, not handling BSYNC -> not compatible with VS1001.
 */


#ifndef VS10XX_H
#define VS10XX_H

#include "board.h"
#include "ui.h"

#define USE_512_BYTE_DREQ 1


/** VS10xx SCI Write Command byte is 0x02 */
#define VS_WRITE_COMMAND 0x02

/** VS10xx SCI Read Command byte is 0x03 */
#define VS_READ_COMMAND 0x03


#define SCI_MODE	0x0   /**< VS10xx register */
#define SCI_STATUS	0x1   /**< VS10xx register */
#define SCI_BASS	0x2   /**< VS10xx register */
#define SCI_CLOCKF	0x3   /**< VS10xx register */
#define SCI_DECODE_TIME	0x4   /**< VS10xx register */
#define SCI_AUDATA	0x5   /**< VS10xx register */
#define SCI_WRAM	0x6   /**< VS10xx register */
#define SCI_WRAMADDR	0x7   /**< VS10xx register */
#define SCI_IN0 0x8
#define SCI_IN1 0x9
#define SCI_AIADDR	0xa   /**< VS10xx register */
#define SCI_VOL		0xb   /**< VS10xx register */
#define SCI_MIXERVOL	0xc   /**< VS10xx register */
#define SCI_ADPCMRECCTL	0xd   /**< VS10xx register */
#define SCI_AICTRL2	0xe   /**< VS10xx register */
#define SCI_AICTRL3	0xf   /**< VS10xx register */


// SCI_MODE register low byte

#define SM_DIFF		0x01   /**< VS10xx register */
#define SM_RECORD_MIXER 0x02   /**< VS10xx register */
#define SM_RESET	0x04   /**< VS10xx register */
#define SM_OUTOFMIDI	0x08   /**< VS10xx register */
#define SM_OUTOFWAV	0x08   /**< VS10xx register */

#define SM_PDOWN	0x10   /**< VS10xx register */
#define SM_TESTS	0x20   /**< VS10xx register */
#define SM_SDI_ADPCM	0x40   /**< VS10xx register */
#define SM_RT_MIDI	0x80   /**< VS10xx register */

// SCI_MODE register high byte

#define SM_DACT		0x01   /**< VS10xx register */
#define SM_SDIORD	0x02   /**< VS10xx register */
#define SM_SDISHARE	0x04   /**< VS10xx register */
#define SM_SDINEW	0x08   /**< VS10xx register */

#define SM_EARSPK_0     0x00
#define SM_EARSPK_1     0x10 
#define SM_EARSPK_2     0x20
#define SM_EARSPK_3     0x30

#define SM_LINE_IN      0x40
#define SM_RECORD_ADPCM 0x80


/** Playing State Global, 0=normal playing, 1=abort playing */
extern xdata unsigned char playingState;

void Vs1103SetMode();



extern bit vs1103_recording_active; ///< VS1103 MODE bit
extern bit vs1103_record_mixer;///< VS1103 MODE bit
extern bit vs1103_use_line_input;///< VS1103 MODE bit
extern bit vs1103_do_reset;///< VS1103 MODE bit
extern bit vs1103_mono_output;///< VS1103 MODE bit
extern bit vs1103_terminate;///< VS1103 MODE bit
extern bit vs1103_earSpkHi;///< VS1103 MODE bit
extern bit vs1103_earSpkLo;///< VS1103 MODE bit
extern bit vs1103_real_time_midi;///< VS1103 MODE bit



/** Execute VS1011/VS1002 Sine Test Function */
void VsSineTest();

void Mp3Reset();

/** Soft Reset of VS10xx (Between songs) */
void Mp3SoftReset();



/** Write VS10xx register*/
#define Mp3WriteRegister(addressbyte,highbyte,lowbyte){ \
 Mp3SelectControl(); \
 SPIPutCharWithoutWaiting(VS_WRITE_COMMAND); \
 SPIPutChar((addressbyte)); \
 SPIPutChar((highbyte)); \
 SPIPutChar((lowbyte)); \
 SPIWait(); \
 Mp3DeselectControl();}

/** Set VS10xx Volume Register */
#define Mp3SetVolume(leftchannel,rightchannel){\
 Mp3WriteRegister(11,(leftchannel),(rightchannel));}

/** Wait for DREQ */
#define Mp3Wait() {while(!MP3_DREQ);}

void Mp3SetMixerVolume(unsigned char midiVolume, 
		       unsigned char waveVolume, 
		       unsigned char micVolume);


/** Read the 16-bit value of a VS10xx register */
unsigned int Mp3ReadRegister (unsigned char addressbyte);

/** Send 2048 zeros. \todo Timeouts for all DREQ busy loop waits! */
void SendZerosToVS10xx();

/** Play disk sectors from disk */
unsigned char PlayDiskSectors (unsigned int nSectorsToPlay);



void AvailableProcessorTime();

/// VS1103B specific function: set MasterTranspose
void Vs1103B_SetMidiTranspose(int transpose);

/// VS1103B specific function: select MIDI channels
void Vs1103B_SetChannelMask(unsigned int newMask);

/// Set MIC analog amplifier value
void Vs1103SetMicGain(unsigned char gain);

/// Terminate wave stream playing
void Vs1103CancelWave();

/// Terminate MIDI stream playing
void Vs1103CancelMidi();


#endif
